@@ -39,7 +39,7 @@
 #include <kconfig.h>
 #include <klocale.h>
 #include <kglobal.h>
-#include <k3process.h>
+#include <kprocess.h>
 #include <kdialog.h>
 #include <kgenericfactory.h>
 #include <khbox.h>
@@ -67,7 +67,7 @@ K_EXPORT_PLUGIN(KGammaConfigFactory("kcmkgamma"))
 
 
 KGamma::KGamma(QWidget* parent_P, const QVariantList &)
-    :KCModule(KGammaConfigFactory::componentData(), parent_P)
+    :KCModule(KGammaConfigFactory::componentData(), parent_P), rootProcess(0)
 {
   bool ok;
   GammaCorrection = true;
@@ -93,7 +93,7 @@ KGamma::KGamma(QWidget* parent_P, const QVariantList &)
       }
       xv->setScreen(currentScreen);
 
-      rootProcess = new K3Process;
+      rootProcess = new KProcess;
       setupUI();
       saved = false;
 
@@ -360,13 +360,13 @@ void KGamma::save() {
       KConfigGroup x86group = config->group("ConfigFile");
       x86group.writeEntry("use", "XF86Config");
 
-      if ( !rootProcess->isRunning() ) {
+      if ( !(rootProcess->state()==QProcess::Running) ) {
         QString Arguments = "xf86gammacfg ";
         for (int i = 0; i < ScreenCount; i++)
           Arguments += rgamma[assign[i]] + ' ' + ggamma[assign[i]] + ' ' + \
                        bgamma[assign[i]] + ' ';
-        rootProcess->clearArguments();
-        *rootProcess << KStandardDirs::findExe("kdesu") << Arguments;
+        rootProcess->clearProgram();
+        rootProcess->setProgram( KStandardDirs::findExe("kdesu"), Arguments.split(' '));
         rootProcess->start();
       }
     }
@@ -399,8 +399,9 @@ bool KGamma::loadSettings() {
   delete config;
 
   if ( ConfigFile == "XF86Config" ) {  // parse XF86Config
-    xf86cfgbox->setChecked(true);
-    return( loadSystemSettings() );
+    bool validGlobalConfig = loadSystemSettings();
+    xf86cfgbox->setChecked( validGlobalConfig );
+    return( validGlobalConfig );
   }
   else { //get gamma settings from user config
     return( loadUserSettings() );
@@ -433,7 +434,7 @@ bool KGamma::loadSystemSettings() {
     int sn = 0;
     bool gm = false;
 
-    // Analyse Screen<->Monitor assignments of multi-head configurations
+    // Analyze Screen<->Monitor assignments of multi-head configurations
     while ( !t.atEnd() ) {
       s = (t.readLine()).simplified();
       QStringList words = s.split(' ');
@@ -492,20 +493,22 @@ bool KGamma::loadSystemSettings() {
         }
       }
       // Extract gamma values
-      for ( int i = 0; i < ScreenCount; i++) {
-        rgamma[i] = ggamma[i] = bgamma[i] = "";
+      if (gm) {
+        for ( int i = 0; i < ScreenCount; i++) {
+          rgamma[i] = ggamma[i] = bgamma[i] = "";
 
-        QStringList words = Gamma[assign[i]].split(' ');
-        QStringList::ConstIterator it = words.begin();
-        if ( words.size() < 4 )
-          rgamma[i] = ggamma[i] = bgamma[i] = *(++it);   // single gamma value
-        else  {
-          rgamma[i] = *(++it);  // eventually rgb gamma values
-          ggamma[i] = *(++it);
-          bgamma[i] = *(++it);
-         }
-       }
+          QStringList words = Gamma[assign[i]].split(' ');
+          QStringList::ConstIterator it = words.constBegin();
+          if ( words.size() < 4 )
+            rgamma[i] = ggamma[i] = bgamma[i] = *(++it);   // single gamma value
+          else  {
+            rgamma[i] = *(++it);  // eventually rgb gamma values
+            ggamma[i] = *(++it);
+            bgamma[i] = *(++it);
+          }
+        }
       }
+    }
   }
   return( validateGammaValues() );
 }
