@@ -4,10 +4,10 @@
  * http://www.kipi-plugins.org
  *
  * Date        : 2006-09-13
- * Description : dcraw settings widgets
+ * Description : LibRaw settings widgets
  *
- * Copyright (C) 2006-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
- * Copyright (C) 2006-2008 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
+ * Copyright (C) 2006-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2006-2009 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
  * Copyright (C) 2007-2008 by Guillaume Castagnino <casta at xwing dot info>
  *
  * This program is free software; you can redistribute it
@@ -22,6 +22,9 @@
  *
  * ============================================================ */
 
+#include "dcrawsettingswidget.h"
+#include "dcrawsettingswidget.moc"
+
 // Qt includes.
 
 #include <QCheckBox>
@@ -43,8 +46,6 @@
 #include "kdcraw.h"
 #include "rnuminput.h"
 #include "rcombobox.h"
-#include "dcrawsettingswidget.h"
-#include "dcrawsettingswidget.moc"
 
 namespace KDcrawIface
 {
@@ -55,6 +56,7 @@ public:
 
     DcrawSettingsWidgetPriv()
     {
+        autoBrightnessBox              = 0;
         sixteenBitsImage               = 0;
         fourColorCheckBox              = 0;
         brightnessLabel                = 0;
@@ -120,6 +122,7 @@ public:
     QCheckBox       *blackPointCheckBox;
     QCheckBox       *whitePointCheckBox;
     QCheckBox       *sixteenBitsImage;
+    QCheckBox       *autoBrightnessBox;
     QCheckBox       *fourColorCheckBox;
     QCheckBox       *dontStretchPixelsCheckBox;
     QCheckBox       *enableNoiseReduction;
@@ -148,15 +151,15 @@ public:
 };
 
 DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, int advSettings)
-                   : QToolBox(parent)
+                   : QToolBox(parent), d(new DcrawSettingsWidgetPriv)
 {
     setup(advSettings);
 }
 
-DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption, 
-                                         bool outputColorSpaceOption, 
+DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption,
+                                         bool outputColorSpaceOption,
                                          bool postProcessingOptions)
-                   : QToolBox(parent)
+                   : QToolBox(parent), d(new DcrawSettingsWidgetPriv)
 {
     int advSettings = 0;
 
@@ -169,8 +172,6 @@ DcrawSettingsWidget::DcrawSettingsWidget(QWidget *parent, bool sixteenBitsOption
 
 void DcrawSettingsWidget::setup(int advSettings)
 {
-    d = new DcrawSettingsWidgetPriv;
-
     // ---------------------------------------------------------------
     // DEMOSAICING Settings panel
 
@@ -272,6 +273,7 @@ void DcrawSettingsWidget::setup(int advSettings)
                                                      "interpolation to Red-Green and Blue-Green channels."));
     demosaicingLayout->addWidget(d->medianFilterPassesLabel,   line, 0, 1, 1);
     demosaicingLayout->addWidget(d->medianFilterPassesSpinBox, line, 1, 1, 2);
+    demosaicingLayout->setRowStretch(line + 1, 10);
 
     insertItem(DEMOSAICING, d->demosaicingSettings, i18n("Demosaicing"));
 
@@ -289,13 +291,13 @@ void DcrawSettingsWidget::setup(int advSettings)
     d->whiteBalanceComboBox->insertItem(RawDecodingSettings::CUSTOM, i18n("Manual"));
     d->whiteBalanceComboBox->setDefaultIndex(RawDecodingSettings::CAMERA);
     d->whiteBalanceComboBox->setWhatsThis(i18n("<p><b>White Balance</b><p>"
-                                          "Configure the raw white balance :<p>"
-                                          "<b>Default D65</b>: Use a standard daylight D65 white balance (dcraw defaults)<p>"
+                                          "Configure the raw white balance:<p>"
+                                          "<b>Default D65</b>: Use a standard daylight D65 white balance.<p>"
                                           "<b>Camera</b>: Use the white balance specified by the camera. "
-                                          "If not available, reverts to default neutral white balance<p>"
+                                          "If not available, reverts to default neutral white balance.<p>"
                                           "<b>Automatic</b>: Calculates an automatic white balance "
-                                          "averaging the entire image<p>"
-                                          "<b>Manual</b>: Set a custom temperature and green level values"));
+                                          "averaging the entire image.<p>"
+                                          "<b>Manual</b>: Set a custom temperature and green level values."));
 
     d->customWhiteBalanceSpinBox = new RIntNumInput(d->whiteBalanceSettings);
     d->customWhiteBalanceSpinBox->setRange(2000, 12000, 10);
@@ -317,7 +319,7 @@ void DcrawSettingsWidget::setup(int advSettings)
     d->unclipColorComboBox = new RComboBox(d->whiteBalanceSettings);
     d->unclipColorComboBox->insertItem(0, i18n("Solid white"));
     d->unclipColorComboBox->insertItem(1, i18n("Unclip"));
-    d->unclipColorComboBox->insertItem(2, i18n("Blend")); 
+    d->unclipColorComboBox->insertItem(2, i18n("Blend"));
     d->unclipColorComboBox->insertItem(3, i18n("Rebuild"));
     d->unclipColorComboBox->setDefaultIndex(0);
     d->unclipColorComboBox->setWhatsThis(i18n("<p><b>Highlights</b><p>"
@@ -339,6 +341,10 @@ void DcrawSettingsWidget::setup(int advSettings)
                                              "Specify the reconstruct highlight level. "
                                              "Low values favor whites and high values favor colors."));
 
+    d->autoBrightnessBox = new QCheckBox(i18n("Auto Brightness"), d->whiteBalanceSettings);
+    d->autoBrightnessBox->setWhatsThis(i18n("<p>If disable, use a fixed white level "
+                                            "and ignore the image histogram to adjust brightness."));
+
     d->brightnessLabel   = new QLabel(i18n("Brightness:"), d->whiteBalanceSettings);
     d->brightnessSpinBox = new RDoubleNumInput(d->whiteBalanceSettings);
     d->brightnessSpinBox->setDecimals(2);
@@ -396,12 +402,14 @@ void DcrawSettingsWidget::setup(int advSettings)
     whiteBalanceLayout->addWidget(d->unclipColorComboBox,            3, 1, 1, 2);
     whiteBalanceLayout->addWidget(d->reconstructLabel,               4, 0, 1, 1);
     whiteBalanceLayout->addWidget(d->reconstructSpinBox,             4, 1, 1, 2);
-    whiteBalanceLayout->addWidget(d->brightnessLabel,                5, 0, 1, 1);
-    whiteBalanceLayout->addWidget(d->brightnessSpinBox,              5, 1, 1, 2);
-    whiteBalanceLayout->addWidget(d->blackPointCheckBox,             6, 0, 1, 1);
-    whiteBalanceLayout->addWidget(d->blackPointSpinBox,              6, 1, 1, 2);
-    whiteBalanceLayout->addWidget(d->whitePointCheckBox,             7, 0, 1, 1);
-    whiteBalanceLayout->addWidget(d->whitePointSpinBox,              7, 1, 1, 2);
+    whiteBalanceLayout->addWidget(d->autoBrightnessBox,              5, 0, 1, 2);
+    whiteBalanceLayout->addWidget(d->brightnessLabel,                6, 0, 1, 1);
+    whiteBalanceLayout->addWidget(d->brightnessSpinBox,              6, 1, 1, 2);
+    whiteBalanceLayout->addWidget(d->blackPointCheckBox,             7, 0, 1, 1);
+    whiteBalanceLayout->addWidget(d->blackPointSpinBox,              7, 1, 1, 2);
+    whiteBalanceLayout->addWidget(d->whitePointCheckBox,             8, 0, 1, 1);
+    whiteBalanceLayout->addWidget(d->whitePointSpinBox,              8, 1, 1, 2);
+    whiteBalanceLayout->setRowStretch(9, 10);
     whiteBalanceLayout->setSpacing(KDialog::spacingHint());
     whiteBalanceLayout->setMargin(KDialog::spacingHint());
 
@@ -475,7 +483,7 @@ void DcrawSettingsWidget::setup(int advSettings)
     d->inputColorSpaceComboBox->setWhatsThis(i18n("<p><b>Camera Profile</b><p>"
                 "Select here the input color space used to decode RAW data.<p>"
                 "<b>None</b>: no input color profile is used during RAW decoding.<p>"
-                "<b>Embedded</b>: use embedded color profile from RAW file if exist.<p>"
+                "<b>Embedded</b>: use embedded color profile from RAW file, if it exists.<p>"
                 "<b>Custom</b>: use a custom input color space profile."));
 
     d->inIccUrlEdit = new KUrlRequester(d->colormanSettings);
@@ -495,7 +503,7 @@ void DcrawSettingsWidget::setup(int advSettings)
                 "Select here the output color space used to decode RAW data.<p>"
                 "<b>Raw (linear)</b>: in this mode, no output color space is used "
                 "during RAW decoding.<p>"
-                "<b>sRGB</b>: this is a RGB color space, created "
+                "<b>sRGB</b>: this is an RGB color space, created "
                 "cooperatively by Hewlett-Packard and Microsoft. It is the "
                 "best choice for images destined for the Web and portrait photography.<p>"
                 "<b>Adobe RGB</b>: this color space is an extended RGB color space, developed by "
@@ -600,6 +608,9 @@ void DcrawSettingsWidget::setup(int advSettings)
     connect(d->sixteenBitsImage, SIGNAL(toggled(bool)),
             this, SIGNAL(signalSettingsChanged()));
 
+    connect(d->autoBrightnessBox, SIGNAL(toggled(bool)),
+            this, SIGNAL(signalSettingsChanged()));
+
     connect(d->fourColorCheckBox, SIGNAL(toggled(bool)),
             this, SIGNAL(signalSettingsChanged()));
 
@@ -875,6 +886,18 @@ void DcrawSettingsWidget::setFourColor(bool b)
 
 // ---------------------------------------------------------------
 
+bool DcrawSettingsWidget::useAutoBrightness()
+{
+    return d->autoBrightnessBox->isChecked();
+}
+
+void DcrawSettingsWidget::setAutoBrightness(bool b)
+{
+    d->autoBrightnessBox->setChecked(b);
+}
+
+// ---------------------------------------------------------------
+
 int DcrawSettingsWidget::unclipColor()
 {
     switch(d->unclipColorComboBox->currentIndex())
