@@ -6,8 +6,8 @@
  * Date        : 2006-09-15
  * Description : Exiv2 library interface for KDE
  *
- * Copyright (C) 2006-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
- * Copyright (C) 2006-2008 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
+ * Copyright (C) 2006-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2006-2009 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -32,21 +32,20 @@ extern "C"
 // Local includes.
 
 #include "version.h"
-#include "kexiv2private.h"
+#include "kexiv2_p.h"
 #include "kexiv2.h"
 
 namespace KExiv2Iface
 {
 
 KExiv2::KExiv2()
+      : d(new KExiv2Priv)
 {
-    d = new KExiv2Priv;
 }
 
 KExiv2::KExiv2(const KExiv2& metadata)
+      : d(new KExiv2Priv)
 {
-    d = new KExiv2Priv;
-
     // No need to use QT containers transormations here. We can use original objects directly.
     d->imageComments = metadata.d->imageComments;
     d->exifMetadata  = metadata.d->exifMetadata;
@@ -60,8 +59,9 @@ KExiv2::KExiv2(const KExiv2& metadata)
 }
 
 KExiv2::KExiv2(const QString& filePath)
+      : d(new KExiv2Priv)
+
 {
-    d = new KExiv2Priv;
     load(filePath);
 }
 
@@ -153,6 +153,14 @@ bool KExiv2::supportMetadataWritting(const QString& typeMime)
         return false;
 #endif
     }
+    else if (typeMime == QString("image/pgf"))
+    {
+#if (EXIV2_TEST_VERSION(0,19,0))
+        return true;
+#else
+        return false;
+#endif
+    }
 
     return false;
 }
@@ -308,20 +316,22 @@ bool KExiv2::save(const QString& filePath) const
     // TIFF/EP Raw file based supported by Exiv2 0.18 are : DNG, NEF, PEF.
     QString rawTiffBasedSupported("dng nef pef");
     QString rawTiffBasedNotSupported("3fr arw cr2 dcr erf k25 kdc mos orf raw sr2 srf");
-    if (rawTiffBasedNotSupported.contains(finfo.suffix().toUpper()))
+    QString ext = finfo.suffix().toLower();
+    if (rawTiffBasedNotSupported.contains(ext))
     {
-        kDebug(51003) << "'" << dinfo.filePath().toAscii().constData() 
+        kDebug(51003) << "'" << finfo.fileName().toAscii().constData() 
                       << "' is TIFF based RAW file not yet supported. Metadata not saved." << endl;
         return false;
     }
 
-    if (rawTiffBasedSupported.contains(finfo.suffix().toUpper()) && !d->writeRawFiles)
+    if (rawTiffBasedSupported.contains(finfo.suffix().toLower()) && !d->writeRawFiles)
     {
-        kDebug(51003) << "'" << dinfo.filePath().toAscii().constData() 
+        kDebug(51003) << "'" << finfo.fileName().toAscii().constData() 
                       << "' is TIFF based RAW file supported but writing mode is disabled. " 
                       << "Metadata not saved." << endl;
         return false;
     }
+    kDebug(51003) << "File Extension: " << ext << " is supported for writing mode" << endl;
 
     try
     {
@@ -336,8 +346,7 @@ bool KExiv2::save(const QString& filePath) const
         // Image Comments ---------------------------------
 
         mode = image->checkMode(Exiv2::mdComment);
-        if (!d->imageComments.empty() && 
-            (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite))
+        if (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite)
         {
             image->setComment(d->imageComments);
         }
@@ -345,8 +354,7 @@ bool KExiv2::save(const QString& filePath) const
         // Exif metadata ----------------------------------
 
         mode = image->checkMode(Exiv2::mdExif);
-        if (!d->exifMetadata.empty() && 
-            (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite))
+        if (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite)
         {
             if (image->mimeType() == "image/tiff")
             {
@@ -390,8 +398,7 @@ bool KExiv2::save(const QString& filePath) const
         // Iptc metadata ----------------------------------
 
         mode = image->checkMode(Exiv2::mdIptc);
-        if (!d->iptcMetadata.empty() && 
-            (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite))
+        if (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite)
         {
             image->setIptcData(d->iptcMetadata);
         }
@@ -401,25 +408,31 @@ bool KExiv2::save(const QString& filePath) const
         // Xmp metadata -----------------------------------
 
         mode = image->checkMode(Exiv2::mdXmp);
-        if (!d->xmpMetadata.empty() && 
-            (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite))
+        if (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite)
         {
             image->setXmpData(d->xmpMetadata);
         }
 
 #endif // _XMP_SUPPORT_
 
-        // NOTE: Don't touch access and modification timestamp of file.
-        struct stat st;
-        ::stat(QFile::encodeName(filePath), &st);
+        if (!d->updateFileTimeStamp)
+        {
+            // NOTE: Don't touch access and modification timestamp of file.
+            struct stat st;
+            ::stat(QFile::encodeName(filePath), &st);
 
-        struct utimbuf ut;
-        ut.modtime = st.st_mtime;
-        ut.actime  = st.st_atime;
+            struct utimbuf ut;
+            ut.modtime = st.st_mtime;
+            ut.actime  = st.st_atime;
 
-        image->writeMetadata();
+            image->writeMetadata();
 
-        ::utime(QFile::encodeName(filePath), &ut);
+            ::utime(QFile::encodeName(filePath), &ut);
+        }
+        else
+        {
+            image->writeMetadata();
+        }
 
         return true;
     }
@@ -434,7 +447,10 @@ bool KExiv2::save(const QString& filePath) const
 bool KExiv2::applyChanges() const
 {
     if (d->filePath.isEmpty())
+    {
+        kDebug(51003) << "Failed to apply changes: file path is empty!" << endl;
         return false;
+    }
 
     return save(d->filePath);
 }
@@ -467,6 +483,16 @@ bool KExiv2::writeRawFiles() const
     return d->writeRawFiles;
 }
 
+void KExiv2::setUpdateFileTimeStamp(bool on)
+{
+    d->updateFileTimeStamp = on;
+}
+
+bool KExiv2::updateFileTimeStamp() const
+{
+    return d->updateFileTimeStamp;
+}
+
 bool KExiv2::setProgramId(bool /*on*/) const
 {
     return true;
