@@ -6,30 +6,33 @@
  * Date        : 2004-02-01
  * Description : plugin loader
  *
- * Copyright (C) 2004-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2004-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2009 by Andi Clemens <andi dot clemens at gmx dot net>
  * Copyright (C) 2004-2005 by Renchi Raju <renchi.raju at kdemail.net>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
  * Public License as published by the Free Software Foundation;
  * either version 2, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * ============================================================ */
 
 /** @file pluginloader.cpp */
 
+#include "pluginloader.h"
+#include "pluginloader.moc"
+
 // Qt include.
 
 #include <QStringList>
-#include <QCheckBox>
-#include <QLayout> 
-#include <QList> 
-#include <QVariantList> 
+#include <QLayout>
+#include <QList>
+#include <QVariantList>
 
 // KDE includes.
 
@@ -45,8 +48,6 @@
 
 #include "plugin.h"
 #include "interface.h"
-#include "pluginloader.h"
-#include "pluginloader.moc"
 
 /**
    \author Renchi Raju
@@ -117,16 +118,16 @@
   PluginLoader::configWidget(), and insert the widget into your normal
   configuration dialog.
 */
-namespace KIPI 
+namespace KIPI
 {
 
 //---------------------------------------------------------------------
 
-struct PluginLoader::Info::Private 
+struct PluginLoader::Info::Private
 {
+    bool          m_shouldLoad;
     KService::Ptr m_service;
-    Plugin* m_plugin;
-    bool m_shouldLoad;
+    Plugin*       m_plugin;
 };
 
 PluginLoader::Info::Info(const KService::Ptr& service, bool shouldLoad)
@@ -186,56 +187,63 @@ void PluginLoader::Info::setShouldLoad(bool value)
 
 static PluginLoader* s_instance = 0;
 
-struct PluginLoader::Private
+class PluginLoaderPrivate
 {
-    PluginList  m_pluginList;
-    Interface*  m_interface;
-    QStringList m_ignores;
+public:
+
+    PluginLoaderPrivate()
+    {
+        m_interface = 0;
+    };
+
+    QStringList               m_ignores;
+    PluginLoader::PluginList  m_pluginList;
+    Interface*                m_interface;
 };
 
 PluginLoader::PluginLoader( const QStringList& ignores, Interface* interface )
+            : d(new PluginLoaderPrivate)
 {
     Q_ASSERT( s_instance == 0 );
-    s_instance = this;
-
-    d = new Private;
-    d->m_interface = interface;
-    d->m_ignores   = ignores;
-
+    s_instance                  = this;
+    d->m_interface              = interface;
+    d->m_ignores                = ignores;
     const KService::List offers = KServiceTypeTrader::self()->query("KIPI/Plugin");
     KSharedConfigPtr config     = KGlobal::config();
     KConfigGroup group          = config->group( QString::fromLatin1( "KIPI/EnabledPlugin" ) );
 
     KService::List::ConstIterator iter;
-    for(iter = offers.begin(); iter != offers.end(); ++iter) 
+    for(iter = offers.begin(); iter != offers.end(); ++iter)
     {
-        KService::Ptr service = *iter;
-        QString name    = service->name();
-        QString comment = service->comment();
-        QString library = service->library();
+        KService::Ptr service   = *iter;
+        QString name            = service->name();
+        QString comment         = service->comment();
+        QString library         = service->library();
         QStringList reqFeatures = service->property( QString::fromLatin1( "X-KIPI-ReqFeatures" ) ).toStringList();
 
-        if (library.isEmpty() || name.isEmpty() ) 
+        if (library.isEmpty() || name.isEmpty() )
         {
-            kWarning( 51001 ) << "KIPI::PluginLoader: Plugin had an empty name or library file - this should not happen.";
+            kWarning( 51001 ) << "KIPI::PluginLoader: Plugin had an empty name or library file - this should not happen." << endl;
             continue;
         }
 
-        if ( d->m_ignores.contains( name ) ) 
+        if ( d->m_ignores.contains( name ) )
         {
-            kDebug( 51001 ) << "KIPI::PluginLoader: plugin " << name << " is in the ignore list for host application";
+            kDebug( 51001 ) << "KIPI::PluginLoader: plugin " << name 
+                            << " is in the ignore list for host application" << endl;
             continue;
         }
 
-        bool appHasAllReqFeatures=true;
-        for( QStringList::Iterator featureIt = reqFeatures.begin(); featureIt != reqFeatures.end(); ++featureIt ) 
+        bool appHasAllReqFeatures = true;
+        for( QStringList::const_iterator featureIt = reqFeatures.constBegin(); 
+             featureIt != reqFeatures.constEnd(); ++featureIt )
         {
-            if ( !d->m_interface->hasFeature( *featureIt ) ) 
+            if ( !d->m_interface->hasFeature( *featureIt ) )
             {
                 kDebug( 51001 ) << "Plugin " << name << " was not loaded because the host application is missing\n"
-                                 << "the feature " << *featureIt << endl;
-				appHasAllReqFeatures=false;
-				break;
+                                << "the feature " << *featureIt << endl;
+                appHasAllReqFeatures = false;
+                break;
             }
         }
 
@@ -256,7 +264,7 @@ PluginLoader::~PluginLoader()
 
 void PluginLoader::loadPlugins()
 {
-    for( PluginList::Iterator it = d->m_pluginList.begin(); it != d->m_pluginList.end(); ++it ) 
+    for( PluginList::Iterator it = d->m_pluginList.begin(); it != d->m_pluginList.end(); ++it )
     {
         loadPlugin( *it );
     }
@@ -271,15 +279,15 @@ void PluginLoader::loadPlugin( Info* info )
         Plugin *plugin = info->service()->createInstance<Plugin>(d->m_interface, QVariantList(), &error);
         if (plugin)
         {
-            kDebug( 51001 ) << "KIPI::PluginLoader: Loaded plugin " << plugin->objectName();
+            kDebug( 51001 ) << "KIPI::PluginLoader: Loaded plugin " << plugin->objectName() << endl;
         }
         else
         {
             kWarning( 51001 ) << "KIPI::PluginLoader:: createInstance returned 0 for "
-                               << info->name()
-                               << " (" << info->library() << ")"
-                               << " with error: "
-                               << error << endl;
+                              << info->name()
+                              << " (" << info->library() << ")"
+                              << " with error: "
+                              << error << endl;
         }
         info->setPlugin(plugin);
     }
@@ -294,54 +302,54 @@ const PluginLoader::PluginList& PluginLoader::pluginList()
 
 PluginLoader* PluginLoader::instance()
 {
-    Q_ASSERT( s_instance != 0);
+    if(!s_instance)
+        kDebug( 51001 ) << "KIPI::PluginLoader::instance is null..." << endl;
+
     return s_instance;
 }
 
 //---------------------------------------------------------------------
 
-ConfigWidget* PluginLoader::configWidget( QWidget* parent )
+ConfigWidget* PluginLoader::configWidget(QWidget* parent)
 {
-    return new ConfigWidget( parent );
+    return new ConfigWidget(parent);
 }
 
-class PluginCheckBox : public QCheckBox
+class PluginCheckBox : public QListWidgetItem
 {
 public:
 
-    PluginCheckBox( PluginLoader::Info* info, QWidget* parent )
-        : QCheckBox( info->comment(), parent ), info( info )
-        {
-            setChecked( info->shouldLoad() );
-        }
+    PluginCheckBox(PluginLoader::Info* info, QListWidget* parent)
+                               : QListWidgetItem(parent, QListWidgetItem::UserType), info(info)
+    {
+        setText(QString("%1  (%2)").arg(info->name(), info->comment()));
+        setFlags(Qt::ItemIsUserCheckable|Qt::ItemIsEnabled);
+        setCheckState(info->shouldLoad() ? Qt::Checked : Qt::Unchecked);
+    }
     PluginLoader::Info* info;
 };
 
-struct ConfigWidget::Private
+class ConfigWidgetPrivate
 {
+public:
+
+    ConfigWidgetPrivate(){};
+
     QList< PluginCheckBox* > _boxes;
 };
 
 ConfigWidget::ConfigWidget(QWidget* parent)
-            : QAbstractScrollArea(parent)
+            : QListWidget(parent), d(new ConfigWidgetPrivate)
 {
-    d = new Private;
-    QWidget* top = new QWidget( viewport() );
-    setViewport( top );
+    setAutoFillBackground(false);
+    setSortingEnabled(true);
+    viewport()->setAutoFillBackground(false);
 
-    QVBoxLayout* lay = new QVBoxLayout(top);
-    lay->setMargin(KDialog::marginHint());
-    lay->setSpacing(KDialog::spacingHint());
-
-    PluginLoader::PluginList list = PluginLoader::instance()->d->m_pluginList;
-    for( PluginLoader::PluginList::Iterator it = list.begin(); it != list.end(); ++it ) 
+    foreach(PluginLoader::Info *info, PluginLoader::instance()->pluginList())
     {
-        PluginCheckBox* cb = new PluginCheckBox( *it, top );
-        lay->addWidget( cb );
-        d->_boxes.append( cb );
+        PluginCheckBox* cb = new PluginCheckBox(info, this);
+        d->_boxes.append(cb);
     }
-
-    lay->addStretch(10);
 }
 
 ConfigWidget::~ConfigWidget()
@@ -352,23 +360,23 @@ ConfigWidget::~ConfigWidget()
 void ConfigWidget::apply()
 {
     KSharedConfigPtr config = KGlobal::config();
-    KConfigGroup group = config->group( QString::fromLatin1( "KIPI/EnabledPlugin" ) );
-    bool changes = false;
+    KConfigGroup group      = config->group( QString::fromLatin1( "KIPI/EnabledPlugin" ) );
+    bool changes            = false;
 
-    for( QList<PluginCheckBox*>::Iterator it = d->_boxes.begin(); it != d->_boxes.end(); ++it ) 
+    for( QList<PluginCheckBox*>::Iterator it = d->_boxes.begin(); it != d->_boxes.end(); ++it )
     {
         bool orig = (*it)->info->shouldLoad();
-        bool load = (*it)->isChecked();
-        if ( orig != load ) 
+        bool load = ((*it)->checkState() == Qt::Checked) ? true : false;
+        if ( orig != load )
         {
             changes = true;
             group.writeEntry( (*it)->info->name(), load );
             (*it)->info->setShouldLoad(load);
-            if ( load ) 
+            if ( load )
             {
                 PluginLoader::instance()->loadPlugin( (*it)->info);
             }
-            else 
+            else
             {
                 if ( (*it)->info->plugin() ) // Do not emit if we had trouble loading plugin.
                     emit PluginLoader::instance()->unplug( (*it)->info);
