@@ -183,6 +183,31 @@ static QString compressedMimeFor( const QString& mime_to_check )
     return QString();
 }
 
+static Part::EmbedMode detectEmbedMode( QWidget *parentWidget, QObject *parent, const QVariantList &args )
+{
+    Q_UNUSED( parentWidget );
+
+    if ( parent
+         && ( parent->objectName() == QLatin1String( "okular::Shell" )
+              || parent->objectName() == QLatin1String( "okular/okular__Shell" ) ) )
+        return Part::NativeShellMode;
+
+    if ( parent
+         && ( QByteArray( "KHTMLPart" ) == parent->metaObject()->className() ) )
+        return Part::KHTMLPartMode;
+
+    Q_FOREACH ( const QVariant &arg, args )
+    {
+        if ( arg.type() == QVariant::String )
+        {
+            if ( arg.toString() == QLatin1String( "Print/Preview" ) )
+                return Part::PrintPreviewMode;
+        }
+    }
+
+    return Part::UnknownEmbedMode;
+}
+
 #undef OKULAR_KEEP_FILE_OPEN
 
 #ifdef OKULAR_KEEP_FILE_OPEN
@@ -198,7 +223,7 @@ QObject *parent,
 const QVariantList &args )
 : KParts::ReadOnlyPart(parent),
 m_tempfile( 0 ), m_fileWasRemoved( false ), m_showMenuBarAction( 0 ), m_showFullScreenAction( 0 ), m_actionsSearched( false ),
-m_cliPresentation(false), m_generatorGuiClient(0), m_keeper( 0 )
+m_cliPresentation(false), m_embedMode(detectEmbedMode(parentWidget, parent, args)), m_generatorGuiClient(0), m_keeper( 0 )
 {
     // first necessary step: copy the configuration from kpdf, if available
     QString newokularconffile = KStandardDirs::locateLocal( "config", "okularpartrc" );
@@ -446,7 +471,7 @@ m_cliPresentation(false), m_generatorGuiClient(0), m_keeper( 0 )
     m_saveAs->setEnabled( false );
 
     QAction * prefs = KStandardAction::preferences( this, SLOT( slotPreferences() ), ac);
-    if ( parent && ( parent->objectName() == QLatin1String( "okular::Shell" ) ) )
+    if ( m_embedMode == NativeShellMode )
     {
         prefs->setText( i18n( "Configure Okular..." ) );
     }
@@ -578,9 +603,9 @@ m_cliPresentation(false), m_generatorGuiClient(0), m_keeper( 0 )
     // ensure history actions are in the correct state
     updateViewActions();
 
-    m_dummyMode = true;
     m_sidebar->setSidebarVisibility( false );
-    if ( !args.contains( QVariant( "Print/Preview" ) ) ) unsetDummyMode();
+    if ( m_embedMode != PrintPreviewMode )
+        unsetDummyMode();
 
 #ifdef OKULAR_KEEP_FILE_OPEN
     m_keeper = new FileKeeper();
@@ -656,7 +681,8 @@ KUrl Part::realUrl() const
 
 void Part::openUrlFromDocument(const KUrl &url)
 {
-    if (m_dummyMode) return;
+    if ( m_embedMode == PrintPreviewMode )
+       return;
 
     m_bExtension->openUrlNotify();
     m_bExtension->setLocationBarUrl(url.prettyUrl());
@@ -809,6 +835,24 @@ QString Part::currentDocument()
 }
 
 
+QString Part::documentMetaData( const QString &metaData ) const
+{
+    const Okular::DocumentInfo * info = m_document->documentInfo();
+    if ( info )
+    {
+        QDomElement docElement = info->documentElement();
+        for ( QDomNode node = docElement.firstChild(); !node.isNull(); node = node.nextSibling() )
+        {
+            const QDomElement element = node.toElement();
+            if ( metaData.compare( element.tagName(), Qt::CaseInsensitive ) == 0 )
+                return element.attribute( "value" );
+        }
+    }
+
+    return QString();
+}
+
+
 bool Part::slotImportPSFile()
 {
     QString app = KStandardDirs::findExe( "ps2pdf" );
@@ -968,8 +1012,30 @@ bool Part::openFile()
     return true;
 }
 
-bool Part::openUrl(const KUrl &url)
+bool Part::openUrl(const KUrl &_url)
 {
+    KUrl url( _url );
+    if ( url.hasHTMLRef() )
+    {
+        const QString dest = url.htmlRef();
+        bool ok = true;
+        const int page = dest.toInt( &ok );
+        if ( ok )
+        {
+            Okular::DocumentViewport vp( page - 1 );
+            vp.rePos.enabled = true;
+            vp.rePos.normalizedX = 0;
+            vp.rePos.normalizedY = 0;
+            vp.rePos.pos = Okular::DocumentViewport::TopLeft;
+            m_document->setNextDocumentViewport( vp );
+        }
+        else
+        {
+            m_document->setNextDocumentDestination( dest );
+        }
+        url.setHTMLRef( QString() );
+    }
+
     // this calls in sequence the 'closeUrl' and 'openFile' methods
     bool openOk = KParts::ReadOnlyPart::openUrl( url );
 
@@ -1046,9 +1112,7 @@ bool Part::closeUrl()
 
 void Part::close()
 {
-    if (parent()
-        && (parent()->objectName() == QLatin1String("okular::Shell")
-            || parent()->objectName() == QLatin1String("okular/okular__Shell")))
+    if ( m_embedMode == NativeShellMode )
     {
         closeUrl();
     }
@@ -1231,8 +1295,10 @@ void Part::slotShowFindBar()
 
 void Part::slotHideFindBar()
 {
-    m_findBar->hide();
-    m_pageView->setFocus();
+    if ( m_findBar->maybeHide() )
+    {
+        m_pageView->setFocus();
+    }
 }
 
 //BEGIN go to page dialog
@@ -1410,9 +1476,10 @@ void Part::slotFindPrev()
 
 void Part::slotSaveFileAs()
 {
-    if (m_dummyMode) return;
+    if ( m_embedMode == PrintPreviewMode )
+       return;
 
-    KUrl saveUrl = KFileDialog::getSaveUrl( url().isLocalFile() ? url().url() : url().fileName(), QString(), widget() );
+    KUrl saveUrl = KFileDialog::getSaveUrl( KUrl("kfiledialog:///okular/" + url().fileName()), QString(), widget() );
     if ( !saveUrl.isValid() || saveUrl.isEmpty() )
         return;
 
@@ -1464,9 +1531,10 @@ void Part::slotSaveFileAs()
 
 void Part::slotSaveCopyAs()
 {
-    if (m_dummyMode) return;
+    if ( m_embedMode == PrintPreviewMode )
+       return;
 
-    KUrl saveUrl = KFileDialog::getSaveUrl( url().isLocalFile() ? url().url() : url().fileName(), QString(), widget() );
+    KUrl saveUrl = KFileDialog::getSaveUrl( KUrl("kfiledialog:///okular/" + url().fileName()), QString(), widget() );
     if ( saveUrl.isValid() && !saveUrl.isEmpty() )
     {
         if ( KIO::NetAccess::exists( saveUrl, KIO::NetAccess::DestinationSide, widget() ) )
@@ -1625,7 +1693,8 @@ void Part::slotPrintPreview()
 
 void Part::slotShowMenu(const Okular::Page *page, const QPoint &point)
 {
-    if (m_dummyMode) return;
+    if ( m_embedMode == PrintPreviewMode )
+       return;
 
     bool reallyShow = false;
     if (!m_actionsSearched)
@@ -1972,9 +2041,8 @@ void Part::psTransformEnded(int exit, QProcess::ExitStatus status)
 
 void Part::unsetDummyMode()
 {
-    if (!m_dummyMode) return;
-
-    m_dummyMode = false;
+    if ( m_embedMode == PrintPreviewMode )
+       return;
 
     m_sidebar->setItemEnabled( 2, true );
     m_sidebar->setItemEnabled( 3, true );
@@ -2090,10 +2158,6 @@ void Part::rebuildBookmarkMenu( bool unplugActions )
         a->setEnabled( false );
         m_bookmarkActions.append( a );
     }
-    for ( int i = 0; i < m_bookmarkActions.count(); ++i )
-    {
-        actionCollection()->addAction( QString( "bookmark_action_%1" ).arg( i ), m_bookmarkActions.at(i) );
-    }
     plugActionList( "bookmarks_currentdocument", m_bookmarkActions );
 
     m_prevBookmark->setEnabled( havebookmarks );
