@@ -79,6 +79,14 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
     Q_INTERFACES(KDocumentViewer)
 
     public:
+        enum EmbedMode
+        {
+            UnknownEmbedMode,
+            NativeShellMode,         // embedded in the native Okular' shell
+            PrintPreviewMode,        // embedded to show the print preview of a document
+            KHTMLPartMode            // embedded in KHTML
+        };
+
         // Default constructor
         Part(QWidget* parentWidget, QObject* parent, const QVariantList& args);
 
@@ -103,6 +111,7 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
         Q_SCRIPTABLE uint pages();
         Q_SCRIPTABLE uint currentPage();
         Q_SCRIPTABLE QString currentDocument();
+        Q_SCRIPTABLE QString documentMetaData( const QString &metaData ) const;
         Q_SCRIPTABLE void slotPreferences();
         Q_SCRIPTABLE void slotFind();
         Q_SCRIPTABLE void slotPrintPreview();
@@ -133,6 +142,7 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
         void slotPreviousBookmark();
         void slotNextBookmark();
         void slotFindNext();
+        void slotFindPrev();
         void slotSaveFileAs();
         void slotSaveCopyAs();
         void slotGetNewStuff();
@@ -227,6 +237,7 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
         KAction *m_selectAll;
         KAction *m_find;
         KAction *m_findNext;
+        KAction *m_findPrev;
         KAction *m_saveAs;
         KAction *m_saveCopyAs;
         KAction *m_printPreview;
@@ -252,7 +263,7 @@ class Part : public KParts::ReadOnlyPart, public Okular::DocumentObserver, publi
         QString m_addBookmarkText;
         QIcon m_addBookmarkIcon;
 
-        bool m_dummyMode;
+        EmbedMode m_embedMode;
 
         KUrl m_realUrl;
 
